@@ -92,29 +92,49 @@ op_inv_spd_full::apply_direct(Mat<typename T1::elem_type>& out, const Base<typen
   const bool allow_approx = has_user_flags && bool(flags & inv_opts::flag_allow_approx);
   const bool likely_sympd = has_user_flags && bool(flags & inv_opts::flag_likely_sympd);
   const bool no_sympd     = has_user_flags && bool(flags & inv_opts::flag_no_sympd    );
+  const bool no_ugly      = has_user_flags && bool(flags & inv_opts::flag_no_ugly     );
   
-  arma_extra_debug_print("op_inv_spd_full: enabled flags:");
+  if(has_user_flags)
+    {
+    arma_extra_debug_print("op_inv_spd_full: enabled flags:");
+    
+    if(tiny        )  { arma_extra_debug_print("tiny");         }
+    if(allow_approx)  { arma_extra_debug_print("allow_approx"); }
+    if(likely_sympd)  { arma_extra_debug_print("likely_sympd"); }
+    if(no_sympd    )  { arma_extra_debug_print("no_sympd");     }
+    if(no_ugly     )  { arma_extra_debug_print("no_ugly");      }
+    
+    if(likely_sympd)  { arma_debug_warn_level(1, "inv_sympd(): option 'likely_sympd' ignored" ); }
+    if(no_sympd)      { arma_debug_warn_level(1, "inv_sympd(): option 'no_sympd' ignored" );     }
+    
+    arma_debug_check( (no_ugly && allow_approx), "inv_sympd(): options 'no_ugly' and 'allow_approx' are mutually exclusive" );
+    }
   
-  if(tiny        )  { arma_extra_debug_print("tiny");         }
-  if(allow_approx)  { arma_extra_debug_print("allow_approx"); }
-  if(likely_sympd)  { arma_extra_debug_print("likely_sympd"); }
-  if(no_sympd    )  { arma_extra_debug_print("no_sympd");     }
-  
-  if(likely_sympd)  { arma_debug_warn_level(1, "inv_sympd(): option 'likely_sympd' ignored" ); }
-  if(no_sympd)      { arma_debug_warn_level(1, "inv_sympd(): option 'no_sympd' ignored" );     }
+  if(no_ugly)
+    {
+    op_inv_spd_state<T> inv_state;
+    
+    const bool status = op_inv_spd_rcond::apply_direct(out, inv_state, expr);
+    
+    if((status == false) || (inv_state.rcond < std::numeric_limits<T>::epsilon()) || arma_isnan(inv_state.rcond))  { return false; }
+    
+    return true;
+    }
   
   if(allow_approx)
     {
-    T rcond = T(0);
+    op_inv_spd_state<T> inv_state;
     
     Mat<eT> tmp;
     
-    const bool status = op_inv_spd_rcond::apply_direct(tmp, rcond, expr);
+    const bool status = op_inv_spd_rcond::apply_direct(tmp, inv_state, expr);
     
-    if((status == false) || (rcond < auxlib::epsilon_lapack(tmp)))
+    if((status == false) || (inv_state.rcond < std::numeric_limits<T>::epsilon()) || arma_isnan(inv_state.rcond))
       {
       const Mat<eT> A = expr.get_ref();
       
+      if(inv_state.is_diag)  { return op_pinv::apply_diag(out, A, T(0)); }
+      
       return op_pinv::apply_sym(out, A, T(0), uword(0));
       }
     
@@ -325,15 +345,15 @@ op_inv_spd_full::apply_tiny_4x4(Mat<eT>& X)
 template<typename T1>
 inline
 bool
-op_inv_spd_rcond::apply_direct(Mat<typename T1::elem_type>& out, typename T1::pod_type& out_rcond, const Base<typename T1::elem_type,T1>& expr)
+op_inv_spd_rcond::apply_direct(Mat<typename T1::elem_type>& out, op_inv_spd_state<typename T1::pod_type>& out_state, const Base<typename T1::elem_type,T1>& expr)
   {
   arma_extra_debug_sigprint();
   
   typedef typename T1::elem_type eT;
   typedef typename T1::pod_type   T;
   
-  out       = expr.get_ref();
-  out_rcond = T(0);
+  out             = expr.get_ref();
+  out_state.rcond = T(0);
   
   arma_debug_check( (out.is_square() == false), "inv_sympd(): given matrix must be square sized" );
   
@@ -355,6 +375,8 @@ op_inv_spd_rcond::apply_direct(Mat<typename T1::elem_type>& out, typename T1::po
     {
     arma_extra_debug_print("op_inv_spd_rcond: detected diagonal matrix");
     
+    out_state.is_diag = true;
+    
     eT* colmem = out.memptr();
     
     T max_abs_src_val = T(0);
@@ -382,7 +404,7 @@ op_inv_spd_rcond::apply_direct(Mat<typename T1::elem_type>& out, typename T1::po
       colmem += N;
       }
     
-    out_rcond = T(1) / (max_abs_src_val * max_abs_inv_val);
+    out_state.rcond = T(1) / (max_abs_src_val * max_abs_inv_val);
     
     return true;
     }
@@ -397,18 +419,18 @@ op_inv_spd_rcond::apply_direct(Mat<typename T1::elem_type>& out, typename T1::po
     
     auxlib::inv_sympd(out, sympd_state);
     
-    if(sympd_state == false)  { out.soft_reset(); out_rcond = T(0); return false; }
+    if(sympd_state == false)  { out.soft_reset(); out_state.rcond = T(0); return false; }
     
-    out_rcond = auxlib::rcond(tmp);
+    out_state.rcond = auxlib::rcond(tmp);
     
-    if(out_rcond == T(0))  { out.soft_reset(); return false; }
+    if(out_state.rcond == T(0))  { out.soft_reset(); return false; }
     
     return true;
     }
   
   bool is_sympd_junk = false;
   
-  return auxlib::inv_sympd_rcond(out, is_sympd_junk, out_rcond, T(-1));
+  return auxlib::inv_sympd_rcond(out, is_sympd_junk, out_state.rcond, T(-1));
   }
 
 
